#!/usr/bin/perl
#
# Adfinis SyGroup AG
# https://git.adfinis-sygroup.ch/ad-sy/adfinis-dirvish.src
#
# Changes expire of an entire vault 

use strict;
use warnings qw( FATAL all );

use POSIX 'strftime';
use Time::ParseDate;
use Getopt::Long;

use constant DIRVISH_MASTER_CONF => '/etc/dirvish/master.conf';

unless ( -f DIRVISH_MASTER_CONF ) {
        die( "/etc/dirvish/master.conf is missing\n" );
}

my $bank = get_bank();
my $vault;
my $new_expire;

if ( ! GetOptions(
                "bank=s"   => \$bank,
                "vault=s"  => \$vault,
                "expire=s" => \$new_expire,
                "help"     => \&help,
        ) ) {
        exit( 1 );
}

unless ( $vault ) {
        die( "No vault given\n" );
}

unless ( $new_expire ) {
        die( "No expire given\n" );
}

my $backup_path = sprintf( "%s/%s", $bank, $vault );

unless ( -d $backup_path ) {
        die( "$backup_path does not exist\n" );
}

print( "Changing the expire time:\n" );
printf( "\tBank: %s\n\tVault: %s\n\tNew expire: %s\n\n", $bank, $vault, $new_expire );

foreach my $image ( glob( "$backup_path/*" ) ) {
        next if ( $image =~ m/\/dirvish$/ );
        next unless ( -d $image );

        change_expire( $image, $new_expire );
}

sub change_expire {
        my $image_path = shift;
        my $new_expire = shift;

        my $summary_path = sprintf( "%s/summary", $image_path );

        my $image_time_now;
        my $image_time_now_ts;

        my $image_expire_old;
        my $image_expire_new;

        my @file_new_lines;

        if ( ! -f $summary_path || -z $summary_path ) {
                printf( "Processing %s\n", $image_path );
                print( "\tNo or empty summary file -- skipping\n" );
                return;
        }

        if ( open( my $fh, '<', $summary_path ) ) {
                foreach my $line ( <$fh> ) {
                        if ( $line =~ m/^Image\-now:\s(.+)/ ) {
                                $image_time_now = $1;
                        }
                        if ( $line =~ m/^Expire:\s(.+)/ ) {
                                $image_expire_old = $1;
                        }
                }

                close( $fh );
        }

        $image_time_now_ts = parsedate( $image_time_now );
        $image_expire_new = sprintf( "%s%s", $new_expire, strftime( ' == %Y-%m-%d %H:%M:%S',
                localtime( parsedate( $new_expire, NOW => $image_time_now_ts ) ) ) );

        printf( "Processing %s\n", $image_path );
        printf( "\tOld expire: %s\n", $image_expire_old );
        printf( "\tNew expire: %s\n", $image_expire_new );

        if ( open ( my $fh, '<', $summary_path ) ) {
                foreach my $line ( <$fh> ) {
                        $line =~ s/^Expire:\s.+/Expire: $image_expire_new/;
                        push( @file_new_lines, $line );
                }

                close( $fh );
        }

        if ( open( my $fh, '>', $summary_path ) ) {
                map { print( $fh $_ ) } @file_new_lines;

                close( $fh );
        }
}

sub get_bank {
        my $bank;
        my $found = 0;

        if ( open( my $fh , '<', DIRVISH_MASTER_CONF ) ) {
                foreach my $line ( <$fh> ) {
                        if ( $line =~ m/^bank:/ ) {
                                $found = 1;
                                next;
                        } else {
                                if ( $found ) {
                                        $bank = $line;
                                        $bank =~ s/\s+//;
                                        chomp( $bank );
                                        last;
                                }
                        }
                }

                close( $fh );
        }

        return $bank;
}

sub help {
        print <<EOF;

Changes the expire date of ALL backup images in a dirvish vault.

USAGE
        dirvish-change-expire.pl {--bank [bank]} --vault [vault] --expire ["new expire time"]

OPTIONS
        --bank bank             The dirvish backup bank.
                                If not specified the bank defined in /etc/dirvish/master.conf is used.
        --vault vault           The backup vault where the expire time should be changed
        --expire "expire time"  The new expire time. IMPORTANT: Use quotes!

EXAMPLES
        dirvish-change-expire.pl --vault kunde01/srtest01 --expire "+15 days"
        dirvish-change-expire.pl --vault kunde02/footest2 --expire "+1 week"

EOF

        exit( 0 );
}
